@@ -91,14 +91,16 @@ sub run {
 
             # detach *before* the ProcManager inits
             $self->daemon_detach if $self->{daemonize};
-
-            $proc_manager->pm_manage;
         }
         elsif ($self->{daemonize}) {
             $self->daemon_detach;
         }
+    } elsif (blessed $self->{manager}) {
+        $proc_manager = $self->{manager};
     }
 
+    $proc_manager && $proc_manager->pm_manage;
+
     while ($request->Accept >= 0) {
         $proc_manager && $proc_manager->pm_pre_dispatch;
 
@@ -296,7 +298,7 @@ Send psgi.errors to STDERR instead of to the FCGI error stream.
 
 =item backlog
 
-Maximum length of the queue of pending connections
+Maximum length of the queue of pending connections, defaults to 100.
 
 =back
 
